// Generated by CoffeeScript 1.6.3
(function() {
  var NewProject, exec, fs, path, pn, spawn;

  fs = require('fs');

  path = require('path');

  pn = path.normalize;

  exec = (require("child_process")).exec;

  spawn = (require("child_process")).spawn;

  module.exports = NewProject = (function() {
    function NewProject(the, cli) {
      var cmd,
        _this = this;
      this.the = the;
      this.cli = cli;
      if (this.cli.argv["new"] === true) {
        console.log("ERROR".bold.red + " You must specify a name for your project");
        return;
      }
      this.pwd = this.the.pwd;
      this.root = this.the.root;
      this.app_skel = path.join(this.the.root, 'cli', 'templates', 'app_skel');
      this.target = path.join(this.pwd, this.cli.argv["new"]);
      if (fs.existsSync(this.target)) {
        console.log("ERROR".bold.red + " Target directory already existis.");
        console.log(("\t" + this.target).yellow);
        return;
      }
      console.log('• Creating app skeleton'.grey);
      cmd = "cp -r " + this.app_skel + " " + this.target;
      exec(cmd, function(error, stdout, stderr) {
        if (error != null) {
          return console.log(error);
        }
        return _this.configure();
      });
    }

    NewProject.prototype.configure = function() {
      var branch, deps, name, repo, src, the_www;
      name = this.cli.argv["new"];
      src = this.cli.argv.src;
      if (src) {
        repo = null;
        branch = null;
        repo = (src.match(/[^#]+/))[0];
        if (!~src.indexOf('/')) {
          repo += '/theoricus';
        }
        repo = "git@github.com:" + repo;
        if (~src.indexOf('#')) {
          branch = (this.cli.argv.src.match(/#(.+)/))[1];
        } else {
          branch = 'master';
        }
        if (this.cli.argv.nogitsub) {
          return this.clone(name, repo, branch);
        } else {
          return this.add_as_submodule(name, repo, branch);
        }
      } else {
        if (!this.cli.argv.dev) {
          deps = "\"theoricus\": \"" + this.the.version + "\"";
          the_www = './node_modules/theoricus/www';
        } else {
          the_www = path.relative(this.target, path.join(this.the.root, 'www'));
          deps = '';
        }
        return this.write_config(name, deps, the_www);
      }
    };

    NewProject.prototype.clone = function(name, repo, branch) {
      var clone, deps, options, params, the_www,
        _this = this;
      console.log('• Cloning theoricus source'.grey);
      deps = '';
      the_www = './vendors/theoricus/www';
      params = ['clone', repo, 'vendors/theoricus'];
      options = {
        cwd: this.target,
        stdio: 'inherit'
      };
      clone = spawn('git', params, options);
      return clone.on('close', function(code) {
        var checkout, cwd;
        if (code > 0) {
          return;
        }
        params = ['checkout', repo];
        cwd = path.join(_this.target, 'vendors', 'theoricus');
        options = {
          cwd: cwd,
          stdio: 'inherit'
        };
        checkout = spawn('git', ['checkout', branch], options);
        return checkout.on('close', function(code) {
          var install;
          if (code > 0) {
            return;
          }
          install = spawn('npm', ['install'], options);
          return install.on('close', function(code) {
            return _this.write_config(name, deps, the_www);
          });
        });
      });
    };

    NewProject.prototype.add_as_submodule = function(name, repo, branch) {
      var deps, init, the_www,
        _this = this;
      console.log('• Cloning theoricus source as submodule'.grey);
      deps = '';
      the_www = './vendors/theoricus/www';
      init = spawn('git', ['init'], {
        cwd: this.target,
        stdio: 'inherit'
      });
      return init.on('close', function(code) {
        var clone, options, params;
        if (code > 0) {
          return;
        }
        params = ['submodule', 'add', repo, 'vendors/theoricus'];
        options = {
          cwd: _this.target,
          stdio: 'inherit'
        };
        clone = spawn('git', params, options);
        return clone.on('close', function(code) {
          var checkout, cwd;
          if (code > 0) {
            return;
          }
          params = ['checkout', branch];
          cwd = path.join(_this.target, 'vendors', 'theoricus');
          options = {
            cwd: cwd,
            stdio: 'inherit'
          };
          checkout = spawn('git', params, options);
          return checkout.on('close', function(code) {
            var install;
            if (code > 0) {
              return;
            }
            install = spawn('npm', ['install'], options);
            return install.on('close', function(code) {
              return _this.write_config(name, deps, the_www);
            });
          });
        });
      });
    };

    NewProject.prototype.write_config = function(name, deps, the_www) {
      var pack, polvo;
      pack = path.join(this.the.root, 'cli', 'templates', 'config', 'package.json');
      pack = fs.readFileSync(pack, 'utf-8');
      pack = pack.replace('~name', name);
      pack = pack.replace('~deps', deps);
      polvo = path.join(this.the.root, 'cli', 'templates', 'config', 'polvo.yml');
      polvo = fs.readFileSync(polvo, 'utf-8');
      polvo = polvo.replace('~theoricus-www', the_www);
      fs.writeFileSync(path.join(this.target, 'package.json'), pack);
      fs.writeFileSync(path.join(this.target, 'polvo.yml'), polvo);
      return this.finish();
    };

    NewProject.prototype.finish = function() {
      var make;
      console.log('• App created, setting up'.grey);
      make = spawn('npm', ['install'], {
        cwd: this.target,
        stdio: 'inherit'
      });
      return make.on('close', function(code) {
        if (code > 0) {
          return;
        }
        console.log("\n\n★  Everything is ready!".cyan);
        console.log("• Remember to add your app details in 'package.json'".grey);
        return console.log('• Have a nice job!\n'.green.grey);
      });
    };

    return NewProject;

  })();

}).call(this);

/*
//@ sourceMappingURL=new_project.map
*/
